import RestClient, { RestClientOpts } from "./RestClient";
import EntityType from "./entities/EntityType";
import PagingResult from "./entities/PagingResult";
import CorporateUser from "./entities/CorporateUser";
import Note from "./entities/Note";
export default class BullhornClient extends RestClient {
    constructor(opts: RestClientOpts);
    /**
     * (FastFind) Searches the following entity types given a string containing search terms:
        ClientContact
        JobOrder
        Candidate
        ClientCorporation
        Lead (if leadsAndOpportunitiesEnabled = true)
        Opportunity (if leadsAndOpportunitiesEnabled = true)

        The results are returned in a flat list with results from each entity type grouped together.

        GET {corpToken}/find?query={query}&countPerEntity={entityMaxResults}
     */
    find(opts: {
        query: string;
        countPerEntity?: number;
        meta?: "off" | "basic" | "full";
        showEditable?: boolean;
    }): Promise<FoundItem[]>;
    /**
     * Find entities by phone number of E.164 format. Will try to find different formats of the phone number until found a result.
     */
    findByPhoneNumber(e164PhoneNumber: string): Promise<FoundItem[]>;
    /**
     * Performs a Lucene search for the following entity types.
     *  ClientContact
     *  ClientCorporation   JobOrder
     *  JobSubmission
     *  Lead
     *  Note
     *  Opportunity
     *  Placement
     *  Task
     *
     * GET {corpToken}/search/{entityType}?query={luceneQueryString}&fields={fieldList}&sort={fieldList} &count={count}&start={start}
     */
    search(type: EntityType, opts: {
        query: string;
        fields: string[];
        sort?: string[];
        count?: number;
        start?: number;
    }): Promise<any>;
    /**
     * Performs a SQL-style query.
     *
     * {corpToken}/query/{entityName}?where={query}&fields={fieldList}&orderBy={fieldList} &count={count}&start={start}
     */
    query(type: EntityType, opts: QueryOptions): Promise<any>;
    getNote(id: number, fields: string[]): Promise<Note>;
    /**
     * Note with same externalID can be created.
     */
    createNote(note: Note): Promise<ChangeResult<Note>>;
    updateNote(id: number, note: Note): Promise<ChangeResult<Note>>;
    deleteNote(id: number): Promise<ChangeResult<void>>;
    /**
     * Retrieve an entity by its id.
     * @param fields See http://bullhorn.github.io/rest-api-docs/?shell#entity-fields
     */
    getEntity(type: EntityType, id: string | number, fields: string[]): Promise<any>;
    createEntity(type: EntityType, entity: any): Promise<any>;
    /**
     * You update entities with a POST request. The URI looks the same as the GET request URI. Place the data comprising the entity fields in JSON format in the request body. The structure of the JSON in a POST request is identical to that returned in HTTP responses to GET requests, but read-only properties cannot be changed.
    You cannot create to-many associations on the entity being updated. You must create to-many associations in a subsequent "associate" call as described in Create to-many associations.
    You can create to-one associations.
    You set association fields by giving as their values a JSON object that contains a field named 'id', and providing the value
    the id of the entity to associate.
    If you provide fields that do not exist on the entity being created, returns an error containing messages about the unknown fields.

    POST {corpToken}/entity/{entityType}/{entityId}

    Returns an HTTP 404 if the requested entity cannot be found, if fields are specified that do not exist on the specified entity, or if values for any mandatory fields with no default value are not supplied.
    */
    updateEntity(type: EntityType, id: number, entity: any): Promise<ChangeResult<any>>;
    deleteEntity(type: EntityType, id: number): Promise<ChangeResult<void>>;
    queryCorporateUsers(opts: QueryOptions): Promise<PagingResult<CorporateUser>>;
    /**
     * Match phone numbers from fields: phone, phone2, phone3, mobile
     * The phone numbers must be in format of E164, NATIONAL, NATIONAL or RFC3966
     */
    queryCorporateUsersByPhoneNumbers(phoneNumbers: string[], opts: QueryOptionsWithoutWhere): Promise<PagingResult<CorporateUser>>;
    /**
     * Get CorporateUsers by name, lastName can be null
     */
    queryCorporateUsersByName(firstName: string, lastName: string, opts: QueryOptionsWithoutWhere): Promise<PagingResult<CorporateUser>>;
    getCandidate(id: string | number, fields: string[]): Promise<any>;
    getEntitlements(entity: EntityType): Promise<any>;
}
export interface QueryOptions extends QueryOptionsWithoutWhere {
    /**
      * SQL-style filter clause see Query where parameter for syntax.
      */
    where: string;
}
export interface QueryOptionsWithoutWhere {
    /**
     * Comma-separated list of field names.
    Note: Use fields=* sparingly. The * value returns all fields, including one-to-many association fields. Each one-to-many field affects performance of the query. Use the /meta/{entityType} call to determine which fields are one-to-many fields.
     */
    fields: string[];
    /**
     * Comma-separated list of field names on which to base the order of returned entities. Precede field name with a minus sign (-) or plus sign (+) to sort results in descending or ascending order based on that field; default is ascending order.
     */
    orderBy?: string[];
    /**
     * Default is off (no meta). Returns metadata that describes the structure of returned entity data.
     */
    meta?: "off" | "basic" | "full";
    /**
     * Default value is false. Whether to show the _editable field in responses. The _editable field indicates whether an entity is editable. Setting showEditable to true results in slower performance; use this setting sparingly and only when needed.
     */
    showEditable?: boolean;
    /**
     * Default value is false. Whether to show the total number of items that match the query.
     */
    showTotalMatched?: boolean;
    /**
     *
     * Limit on the number of entities to return. If the set of matched results is larger than the count value, the returned results is capped at the count value.
    The default count is 15. The maximum count is 500; if you specify a count greater than 500, a message at the end of the response indicates you have specified too many items.
    The response also includes the start value of the request. This is useful when you want to make calls to page additional sets of data.
     */
    count?: number;
    /**
     * From the set of matched results, return item numbers start through (start + count).
     */
    start?: number;
}
/**
 *  Sample:
 *  { entityId: 216311,
    entityType: 'ClientContact',
    title: ' Kevin Conroy',
    byLine: 'Univision Interactive Media',
    location: '' }
 */
export interface FoundItem {
    entityId: number;
    entityType: EntityType;
    title: string;
    byLine: string;
    location: string;
}
export interface ChangeResult<T> {
    changedEntityType: EntityType;
    changedEntityId: number;
    changeType: "INSERT" | "UPDATE" | "DELETE" | "DELETED" | "DISASSOCIATE";
    data?: T;
}
