"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var RestClient_1 = require("./RestClient");
var escape_sql_string_1 = require("./escape-sql-string");
var google_libphonenumber_1 = require("google-libphonenumber");
var BullhornClient = (function (_super) {
    tslib_1.__extends(BullhornClient, _super);
    function BullhornClient(opts) {
        return _super.call(this, opts) || this;
    }
    /**
     * (FastFind) Searches the following entity types given a string containing search terms:
        ClientContact
        JobOrder
        Candidate
        ClientCorporation
        Lead (if leadsAndOpportunitiesEnabled = true)
        Opportunity (if leadsAndOpportunitiesEnabled = true)

        The results are returned in a flat list with results from each entity type grouped together.

        GET {corpToken}/find?query={query}&countPerEntity={entityMaxResults}
     */
    BullhornClient.prototype.find = function (opts) {
        return this.getJson("/find", opts).then(function (res) { return res["data"]; });
    };
    /**
     * Find entities by phone number of E.164 format. Will try to find different formats of the phone number until found a result.
     */
    BullhornClient.prototype.findByPhoneNumber = function (e164PhoneNumber) {
        var _this = this;
        var phoneUtil = google_libphonenumber_1.PhoneNumberUtil.getInstance();
        var phoneNumber;
        try {
            phoneNumber = phoneUtil.parse(e164PhoneNumber);
        }
        catch (e) {
            return Promise.reject(new Error("Fail to parse phone number " + e164PhoneNumber + ", " + e));
        }
        var formatsToFind = [google_libphonenumber_1.PhoneNumberFormat.E164, google_libphonenumber_1.PhoneNumberFormat.NATIONAL]; // PhoneNumberFormat.INTERNATIONAL, PhoneNumberFormat.RFC3966
        var find = function () {
            if (formatsToFind.length < 1) {
                return Promise.resolve([]);
            }
            return _this.find({ query: phoneUtil.format(phoneNumber, formatsToFind.shift()) }).then(function (entities) {
                if (entities.length > 0) {
                    return entities;
                }
                else {
                    return find();
                }
            });
        };
        return find();
    };
    /**
     * Performs a Lucene search for the following entity types.
     *  ClientContact
     *  ClientCorporation   JobOrder
     *  JobSubmission
     *  Lead
     *  Note
     *  Opportunity
     *  Placement
     *  Task
     *
     * GET {corpToken}/search/{entityType}?query={luceneQueryString}&fields={fieldList}&sort={fieldList} &count={count}&start={start}
     */
    BullhornClient.prototype.search = function (type, opts) {
        return this.getJson("/search/" + type, opts);
    };
    /**
     * Performs a SQL-style query.
     *
     * {corpToken}/query/{entityName}?where={query}&fields={fieldList}&orderBy={fieldList} &count={count}&start={start}
     */
    BullhornClient.prototype.query = function (type, opts) {
        var arrFlds = { fields: opts.fields.join(",") };
        if (opts.orderBy) {
            arrFlds["orderBy"] = opts.orderBy.join(",");
        }
        var query = Object.assign({}, opts, arrFlds);
        return this.getJson("/query/" + type, query);
    };
    //=====================================================
    // For notes
    BullhornClient.prototype.getNote = function (id, fields) {
        return this.getEntity("Note", id, fields);
    };
    /**
     * Note with same externalID can be created.
     */
    BullhornClient.prototype.createNote = function (note) {
        return this.createEntity("Note", note);
    };
    BullhornClient.prototype.updateNote = function (id, note) {
        return this.updateEntity("Note", id, note);
    };
    BullhornClient.prototype.deleteNote = function (id) {
        return this.deleteEntity("Note", id);
    };
    //=====================================================
    // For entities.
    /**
     * Retrieve an entity by its id.
     * @param fields See http://bullhorn.github.io/rest-api-docs/?shell#entity-fields
     */
    BullhornClient.prototype.getEntity = function (type, id, fields) {
        return this.getJson("/entity/" + type + "/" + (id || ""), { fields: fields.join(",") }).then(function (res) { return res.data; });
    };
    BullhornClient.prototype.createEntity = function (type, entity) {
        return this.putJson("/entity/" + type, entity);
    };
    /**
     * You update entities with a POST request. The URI looks the same as the GET request URI. Place the data comprising the entity fields in JSON format in the request body. The structure of the JSON in a POST request is identical to that returned in HTTP responses to GET requests, but read-only properties cannot be changed.
    You cannot create to-many associations on the entity being updated. You must create to-many associations in a subsequent "associate" call as described in Create to-many associations.
    You can create to-one associations.
    You set association fields by giving as their values a JSON object that contains a field named 'id', and providing the value
    the id of the entity to associate.
    If you provide fields that do not exist on the entity being created, returns an error containing messages about the unknown fields.

    POST {corpToken}/entity/{entityType}/{entityId}

    Returns an HTTP 404 if the requested entity cannot be found, if fields are specified that do not exist on the specified entity, or if values for any mandatory fields with no default value are not supplied.
    */
    BullhornClient.prototype.updateEntity = function (type, id, entity) {
        return this.postJson("/entity/" + type + "/" + id, entity);
    };
    BullhornClient.prototype.deleteEntity = function (type, id) {
        return this.delete("/entity/" + type + "/" + id).then(function (res) { return res.json(); });
    };
    //=====================================================
    // For entities.
    BullhornClient.prototype.queryCorporateUsers = function (opts) {
        return this.query("CorporateUser", opts);
    };
    /**
     * Match phone numbers from fields: phone, phone2, phone3, mobile
     * The phone numbers must be in format of E164, NATIONAL, NATIONAL or RFC3966
     */
    BullhornClient.prototype.queryCorporateUsersByPhoneNumbers = function (phoneNumbers, opts) {
        if (phoneNumbers.length < 1) {
            return Promise.reject("No phone numbers provided for querying CorporateUser.");
        }
        var fieldsToMatch = ["phone", "phone2", "phone3", "mobile"];
        var formatsToFind = [google_libphonenumber_1.PhoneNumberFormat.E164, google_libphonenumber_1.PhoneNumberFormat.NATIONAL, google_libphonenumber_1.PhoneNumberFormat.NATIONAL, google_libphonenumber_1.PhoneNumberFormat.RFC3966];
        var matches = [];
        var phoneUtil = google_libphonenumber_1.PhoneNumberUtil.getInstance();
        for (var _i = 0, phoneNumbers_1 = phoneNumbers; _i < phoneNumbers_1.length; _i++) {
            var phone = phoneNumbers_1[_i];
            var phoneNumber = phoneUtil.parse(phone);
            for (var _a = 0, fieldsToMatch_1 = fieldsToMatch; _a < fieldsToMatch_1.length; _a++) {
                var fld = fieldsToMatch_1[_a];
                for (var _b = 0, formatsToFind_1 = formatsToFind; _b < formatsToFind_1.length; _b++) {
                    var fmt = formatsToFind_1[_b];
                    matches.push([fld, phoneUtil.format(phoneNumber, fmt)]);
                }
            }
        }
        var queryOpts = opts;
        queryOpts.where = matches.map(function (m) { return m[0] + "=" + escape_sql_string_1.default(m[1]); }).join(" or ");
        return this.queryCorporateUsers(queryOpts);
    };
    /**
     * Get CorporateUsers by name, lastName can be null
     */
    BullhornClient.prototype.queryCorporateUsersByName = function (firstName, lastName, opts) {
        var w = "firstName=" + escape_sql_string_1.default(firstName);
        if (lastName) {
            w += " and lastName=" + escape_sql_string_1.default(lastName);
        }
        var queryOpts = opts;
        queryOpts.where = w;
        return this.query("CorporateUser", queryOpts);
    };
    BullhornClient.prototype.getCandidate = function (id, fields) {
        return this.getEntity("Candidate", id, fields);
    };
    BullhornClient.prototype.getEntitlements = function (entity) {
        return this.getJson("/entitlements/" + entity);
    };
    return BullhornClient;
}(RestClient_1.default));
exports.default = BullhornClient;
