"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fetch = require("isomorphic-fetch");
var querystring = require("querystring");
var url = require("url");
var packageInfo = require("./pkg");
var AUTH_URL = "/oauth/authorize";
var TOKEN_URL = "/oauth/token";
var LOGIN_URL = "/rest-services/login";
var LOGOUT_URL = "/logout";
var API_VERSION = '2.0';
var SESSION_KEY_NAME = "BhRestToken";
var RestClient = (function () {
    function RestClient(opts) {
        this.agents = [packageInfo.name + "/" + packageInfo.version];
        this.server = opts.server;
        this.authServer = opts.authServer;
        this.clientId = opts.clientId;
        this.secret = opts.secret;
        this.sessionTtl = opts.sessionTtl || 6 * 60;
        this.agents.push(this.clientId);
    }
    RestClient.prototype.getUserAgent = function () {
        return this.agents.join(' ');
    };
    /** Require login. */
    RestClient.prototype.getJson = function (path, query) {
        return this.send(path, query).then(function (res) { return res.json(); });
    };
    RestClient.prototype.putJson = function (path, body, query) {
        return this.send(path, query, {
            method: "PUT",
            body: JSON.stringify(body),
            headers: { "Content-Type": "application/json" }
        }).then(function (res) { return res.json(); });
    };
    RestClient.prototype.postJson = function (path, body, query) {
        return this.send(path, query, {
            method: "POST",
            body: JSON.stringify(body),
            headers: { "Content-Type": "application/json" }
        }).then(function (res) { return res.json(); });
    };
    RestClient.prototype.delete = function (path, query) {
        return this.send(path, query, {
            method: "DELETE"
        });
    };
    /**
     * Send an authenticated API http request. If a client request contains an invalid session key, the response status is HTTP 401 (unauthorized)
     * FIXME: Some data like FormData cannot be sent multiple times.
     */
    RestClient.prototype.send = function (endpoint, query, opts) {
        var _this = this;
        if (!this.sessionKey) {
            return Promise.reject(new Error("Authentication required to make API calls."));
        }
        opts = opts || {};
        var headers = opts.headers = opts.headers || {};
        headers[SESSION_KEY_NAME] = this.sessionKey;
        headers["X-User-Agent"] = this.getUserAgent();
        var _url = url.format({ pathname: this.baseUrl + endpoint, query: query });
        return fetch(_url, opts).then(function (res) {
            if (res.ok) {
                return res;
            }
            if (isJsonRes(res)) {
                return res.json().then(function (err) {
                    // Session key is invalid, refresh it.{""errorCode"":401,""errorMessage"":""Bad 'BhRestToken' or timed-out.""}}"
                    if (err.errorCode == 401 && _this.refreshToken) {
                        return _this.refreshSession(_this.refreshToken);
                    }
                    var e = new Error(opts && opts.method + ' ' + _url + ': ' + err["errorMessage"]);
                    e["detail"] = err;
                    throw e;
                }).then(function () {
                    return _this.send(endpoint, query, opts);
                });
            }
            return res.text().then(function (txt) { throw new Error(txt); });
        });
    };
    /**
     * Auth code -> token -> session key.
     * If there is an auth on the progress, second attempt will return the existing promise.
     * If already login success, will throw error.
     */
    RestClient.prototype.auth = function (username, password) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var authCode, token;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.sessionKey) {
                            throw new Error("Already logged in, can not login multiple times.");
                        }
                        return [4 /*yield*/, this.getAuthCode(username, password)];
                    case 1:
                        authCode = _a.sent();
                        return [4 /*yield*/, this.getToken(authCode)];
                    case 2:
                        token = _a.sent();
                        return [4 /*yield*/, this.login(token)];
                    case 3:
                        _a.sent();
                        return [2 /*return*/, this];
                }
            });
        });
    };
    /**
     * The client id and auth code is retrieved from the redirect url of Bullhorn oauth login page.
     * If there is an auth on the progress, second attempt will return the existing promise.
     * If already login success, will throw error.
     **/
    RestClient.prototype.oauth = function (clientId, authCode) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var token;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.sessionKey) {
                            throw new Error("Already logged in, can not login multiple times.");
                        }
                        if (clientId != this.clientId) {
                            throw new Error("Client ID mismatch, stored client ID:" + this.clientId + ", returned client id:" + clientId);
                        }
                        return [4 /*yield*/, this.getToken(authCode)];
                    case 1:
                        token = _a.sent();
                        return [4 /*yield*/, this.login(token)];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /** Get the login page url. */
    RestClient.prototype.oauthLoginPageUrl = function (opts) {
        if (opts === void 0) { opts = {}; }
        var searchParams = querystring.stringify({
            client_id: this.clientId,
            response_type: "code",
            redirect_uri: this.redirectUrl,
            state: opts.state
        });
        return this.authServer + AUTH_URL + "?" + searchParams;
    };
    RestClient.prototype.logout = function () {
        var _this = this;
        return this.getJson(LOGOUT_URL).then(function () {
            _this.sessionKey = undefined;
            _this.baseUrl = undefined;
            _this.refreshToken = undefined;
        });
    };
    /**
     * ttl: Session time-to-live in minutes. Default value is 10 minutes.
     * errorMessage for used or expired access_token: Invalid or expired OAuth access token.
     */
    RestClient.prototype.login = function (token) {
        var _this = this;
        this.auth;
        this.refreshToken = token.refreshToken;
        var loginUrl = this.server + LOGIN_URL + "?"
            + querystring.stringify({ version: API_VERSION, access_token: token.accessToken, ttl: this.sessionTtl });
        return fetch(loginUrl, { headers: { "X-User-Agent": this.getUserAgent() } }).then(function (res) {
            return res.json();
        }).then(function (json) {
            var sessionKey = json[SESSION_KEY_NAME];
            if (!sessionKey) {
                throw new Error(json["errorMessage"]);
            }
            _this.sessionKey = sessionKey;
            _this.baseUrl = json["restUrl"];
        });
    };
    RestClient.prototype.getToken = function (authCode) {
        var tokenUrl = this.authServer + TOKEN_URL + "?" + querystring.stringify({
            grant_type: "authorization_code",
            code: authCode,
            client_id: this.clientId,
            client_secret: this.secret,
            redirect_uri: this.redirectUrl
        });
        return fetch(tokenUrl, { method: "POST", headers: { "X-User-Agent": this.getUserAgent() } })
            .then(function (res) { return res.json(); })
            .then(function (json) {
            if (!json["access_token"]) {
                var e = new Error(json["errorMessage"]);
                e["detail"] = json;
                throw e;
            }
            var token = new Token(json);
            return token;
        });
    };
    /**
     * If current session key is invalid, update it with refreshToken.
     */
    RestClient.prototype.refreshSession = function (refreshToken) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var tokenUrl, res, json, e, token;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tokenUrl = this.authServer + TOKEN_URL + "?" + querystring.stringify({
                            grant_type: "refresh_token",
                            refresh_token: this.refreshToken,
                            client_id: this.clientId,
                            client_secret: this.secret
                        });
                        return [4 /*yield*/, fetch(tokenUrl, { method: "POST", headers: { "X-User-Agent": this.getUserAgent() } })];
                    case 1:
                        res = _a.sent();
                        return [4 /*yield*/, res.json()];
                    case 2:
                        json = _a.sent();
                        if (!json["access_token"]) {
                            e = new Error(json["errorMessage"]);
                            e["detail"] = json;
                            throw e;
                        }
                        token = new Token(json);
                        return [2 /*return*/, this.login(token)];
                }
            });
        });
    };
    /** Returns auth code. */
    RestClient.prototype.getAuthCode = function (username, password) {
        var _this = this;
        var searchParams = querystring.stringify({
            client_id: this.clientId,
            response_type: "code",
            username: username,
            password: password,
            action: "Login",
            redirect_uri: this.redirectUrl
        });
        var authUrl = this.authServer + AUTH_URL + "?" + searchParams;
        return fetch(authUrl, { redirect: "manual", headers: { "X-User-Agent": this.getUserAgent() } }).then(function (res) {
            var redictionUrl = res.headers.get("location");
            if (!redictionUrl) {
                return res.text().then(function (text) {
                    throw new Error("Auth code is in location header, but location header is not present. http status:" + res.status + "\n" + text);
                });
            }
            var q = url.parse(redictionUrl, true).query;
            var authCode = q.code;
            if (authCode && q.client_id == _this.clientId) {
                return authCode;
            }
        });
    };
    return RestClient;
}());
exports.default = RestClient;
var Token = (function () {
    function Token(data) {
        this.accessToken = data["access_token"];
        this.type = data["token_type"];
        this.expiresIn = data["expires_in"];
        this.refreshToken = data["refresh_token"];
    }
    return Token;
}());
exports.Token = Token;
function isJsonRes(res) {
    var ct = res.headers.get("content-type");
    return ct && ct.match("application/json");
}
